@@ -33,16 +33,8 @@ class MySQLQueryProcessor(object):
     :param query:
         MySQL query string.
 
-    :param strict: (optional)
-        If set to True ambiguous columns (b) in queries such as
-            SELECT a, b FROM db.tab1
-            JOIN (
-                SELECT id, col AS b FROM db.tab2
-            ) AS sub USING(id)
-        will raise a ``QueryError``.
-
     """
-    def __init__(self, query=None, strict=False):
+    def __init__(self, query=None):
         self.walker = antlr4.ParseTreeWalker()
 
         self.columns = set()
@@ -50,7 +42,6 @@ class MySQLQueryProcessor(object):
         self.functions = set()
         self.display_columns = []
         self.syntax_error_listener = SyntaxErrorListener()
-        self._strict = strict
         if query is not None:
             self._query = self._strip_query(query)
             self.process_query()
@@ -65,6 +56,9 @@ class MySQLQueryProcessor(object):
         join = 0
         join_using = None
 
+        # Keep track of the ctx stack
+        ctx_stack = []
+
         for i in column_keyword_function_listener.data:
             if isinstance(i[1], MySQLParser.Displayed_columnContext):
                 # this happens if there is an expression involving
@@ -77,38 +71,61 @@ class MySQLQueryProcessor(object):
                 alias = parse_alias(i[1].alias())
                 if alias is not None:
                     column_aliases.append(alias)
+                ctx_stack.append(i)
 
             if isinstance(i[1], MySQLParser.Table_atomContext):
                 select_list_tables.append([i[2], i[0]])
+                ctx_stack.append(i)
 
             if isinstance(i[1], MySQLParser.Table_referencesContext):
                 if len(i) > 2:
                     select_list_table_references.extend(i[2])
+                    ctx_stack.append(i)
 
             if isinstance(i[1], MySQLParser.Select_listContext):
                 if len(i) == 3:
                     select_list_columns.append(i[2])
+                    ctx_stack.append(i)
 
-            if isinstance(i[1], MySQLParser.Where_clauseContext):
+            if isinstance(i[1], MySQLParser.Where_clauseContext) or\
+               isinstance(i[1], MySQLParser.Having_clauseContext):
                 if len(i[2]) > 1:
                     for j in i[2]:
                         other_columns.append([j])
                 else:
                     other_columns.append(i[2])
+                ctx_stack.append(i)
 
             if isinstance(i[1], MySQLParser.Join_conditionContext):
                 join = i[0]
-                if len(i[2]) > 1:
-                    for j in i[2]:
-                        other_columns.append([j])
                 join_using = i[2]
 
+                # if USING we need all columns in all columns if they
+                # have no references
+                if i[1].USING_SYM():
+                    for ctx in ctx_stack[::-1]:
+                        if not isinstance(ctx[1],
+                                          MySQLParser.Table_atomContext):
+                            break
+                        for ju in join_using:
+                            if ju[0][1] is None:
+                                other_columns.append([[[ctx[2][0][0],
+                                                        ctx[2][0][1],
+                                                        ju[0][2]], None]])
+                elif i[1].ON():
+                    if len(i[2]) > 1:
+                        for j in i[2]:
+                            other_columns.append([j])
+
+                ctx_stack.append(i)
+
             if isinstance(i[1], MySQLParser.Orderby_clauseContext):
                 if len(i[2]) > 1:
                     for j in i[2]:
                         go_columns.append([j])
                 else:
                     go_columns.append(i[2])
+                ctx_stack.append(i)
 
             if isinstance(i[1], MySQLParser.Groupby_clauseContext):
                 if len(i[2]) > 1:
@@ -116,6 +133,7 @@ class MySQLQueryProcessor(object):
                         go_columns.append([j])
                 else:
                     go_columns.append(i[2])
+                ctx_stack.append(i)
 
         return select_list_columns, select_list_tables,\
             select_list_table_references, other_columns, go_columns, join,\
@@ -129,7 +147,11 @@ class MySQLQueryProcessor(object):
         column_found = False
 
         for bc in ref:
-            if bc[1] and c[0][2] == bc[1]:
+            if bc[0][2] == '*':
+                t = [[bc[0][0], bc[0][1]], 'None']
+                column_found = True
+                break
+            elif bc[1] and c[0][2] == bc[1]:
                 t = [[bc[0][0], bc[0][1]], 'None']
                 cname = bc[0][2]
                 if c[1] is None:
@@ -150,6 +172,8 @@ class MySQLQueryProcessor(object):
         # Here we store all columns that might have references somewhere
         # higer up in the tree structure. We'll revisit them later.
         missing_columns = []
+        remove_column_idxs = []
+        extra_columns = []
 
         for i, col in enumerate(columns):
             c = col[0]
@@ -157,6 +181,19 @@ class MySQLQueryProcessor(object):
             cname = c[0][2]
             calias = c[1]
 
+            # if * is selected we don't care too much
+            if c[0][0] is None and c[0][1] is None and c[0][2] == '*':
+                for slt in select_list_tables:
+                    extra_columns.append([[slt[0][0][0], slt[0][0][1], cname],
+                                          calias])
+                remove_column_idxs.append(i)
+                continue
+
+            # this can happen for example in ... WHERE EXISTS ... clauses
+            if cname is None and calias is None:
+                remove_column_idxs.append(i)
+                continue
+
             tab = [[None, None], None]
             try:
                 tab = select_list_tables[0][0]
@@ -170,8 +207,8 @@ class MySQLQueryProcessor(object):
                          (tab[1] is None and c[0][1] is None)):
                     cname, calias, column_found, tab =\
                             self._get_budget_column(c, tab, budget[-1][2])
-                    # raise an ambigous column if using strict
-                    if column_found and self._strict and c[0][1] is None:
+                    # raise an ambigous column
+                    if column_found and c[0][1] is None:
                         raise QueryError("Column '%s' is possibly ambiguous."
                                          % c[0][2])
 
@@ -188,8 +225,9 @@ class MySQLQueryProcessor(object):
                     cname, calias, column_found, tab =\
                             self._get_budget_column(c, tab, ref[2])
 
+                    ref_cols = [j[0][2] for j in ref[2]]
                     if not column_found and c[0][1] is not None\
-                            and c[0][1] != tab[0][1]:
+                            and c[0][1] != tab[0][1] and '*' not in ref_cols:
                         raise QueryError("Unknown column '%s.%s'." % (c[0][1],
                                                                       c[0][2]))
 
@@ -198,7 +236,14 @@ class MySQLQueryProcessor(object):
                     tab = ref[0]
 
             except KeyError:
-                if c[0][2] is not None and c[0][1] is not None:
+                if None not in c[0]:
+                    cname = c[0][2]
+                    calias = c[1]
+                    tab = [[c[0][0], c[0][1]]]
+                    column_found = True
+
+                # table is either referenced directly or by an alias
+                elif c[0][2] is not None and c[0][1] is not None:
                     if subquery_contents is not None:
                         try:
                             contents = subquery_contents[c[0][1]]
@@ -206,24 +251,55 @@ class MySQLQueryProcessor(object):
                                 self._get_budget_column(c, tab, contents)
 
                         except KeyError:
+                            tabs = [j[0][0][:2] for j in
+                                    subquery_contents.values()]
+                            tabs += [j[0][0] for j in select_list_tables]
+                            column_found = False
+                            for t in tabs:
+                                if t[1] == c[0][1]:
+                                    cname = c[0][2]
+                                    calias = c[1]
+                                    tab = [t]
+                                    column_found = True
+
+                            if not column_found:
+                                missing_columns.append(c)
+                                columns[i] = [[c[0][0], c[0][1],
+                                               c[0][2]], c[1]]
+                                if touched_columns is not None:
+                                    touched_columns.append([[c[0][0], c[0][1],
+                                                           c[0][2]], c[1]])
+                                continue
+                    else:
+                        if tab[0][1] == c[0][1]:
+                            columns[i] = [[tab[0][0], tab[0][1],
+                                          c[0][2]], c[1]]
+                        else:
+
                             missing_columns.append(c)
                             columns[i] = [[c[0][0], c[0][1], c[0][2]], c[1]]
                             if touched_columns is not None:
                                 touched_columns.append([[c[0][0], c[0][1],
                                                        c[0][2]], c[1]])
-                            continue
-                    else:
-                        missing_columns.append(c)
-                        columns[i] = [[c[0][0], c[0][1], c[0][2]], c[1]]
-                        if touched_columns is not None:
-                            touched_columns.append([[c[0][0], c[0][1],
-                                                   c[0][2]], c[1]])
                         continue
 
-                elif c[0][2] is not None and c[0][1] is None and\
-                        len(ref_dict.keys()) > 1 and not join:
+                elif c[0][2] is not None and c[0][2] != '*' and c[0][1] is \
+                        None and len(ref_dict.keys()) > 1 and not join:
                     raise QueryError("Column '%s' is ambiguous." % c[0][2])
 
+                #  elif c[0][2] is not None and c[0][2] == '*' and c[0][1] is \
+                        #  None and len(ref_dict.keys()) > 1 and not join:
+
+                    #  tv = list(ref_dict.values())
+                    #  try:
+                        #  columns[i] = [[tv[0][0][0][0], tv[0][0][1],
+                                       #  c[0][2]], c[1]]
+                        #  for t in tv[1:]:
+                            #  extra_columns.append([[t[0][0][0], t[0][0][1],
+                                                  #  c[0][2]], c[1]])
+                    #  except TypeError:
+                        #  pass
+
                 elif len(budget) and tab[0][0] is None and tab[0][1] is None:
                     ref = budget[-1]
                     column_found = False
@@ -243,6 +319,10 @@ class MySQLQueryProcessor(object):
             else:
                 columns[i] = [[tab[0][0], tab[0][1], cname], calias]
 
+        for i in remove_column_idxs[::-1]:
+            columns.pop(i)
+
+        columns.extend(extra_columns)
         return missing_columns
 
     def process_query(self, replace_schema_name={}):
@@ -250,10 +330,6 @@ class MySQLQueryProcessor(object):
         Parses and processes the query. After a successful run it fills up
         columns, keywords, functions and syntax_errors lists.
 
-        :param strict: (optional)
-            Dictionary of replacement schema names. If it is provided
-            each key schema name will be replaced with its value.
-
         """
         # Antlr objects
         inpt = antlr4.InputStream(self.query)
@@ -276,6 +352,7 @@ class MySQLQueryProcessor(object):
         subquery_aliases = [None]
         keywords = []
         functions = []
+        tables = []
 
         self.walker.walk(query_listener, tree)
         keywords.extend(query_listener.keywords)
@@ -295,11 +372,6 @@ class MySQLQueryProcessor(object):
 
         subquery_contents = {}
 
-        # Check if we have non-unique subquery aliases
-        if len(set(subquery_aliases.values())) !=\
-                len(subquery_aliases.values()):
-            raise QueryError('Query includes non-unique subquery aliases.')
-
         # Iterate through subqueries starting with the lowerst level
         for ccc, ctx in enumerate(query_listener.select_expressions[::-1]):
             remove_subquieries_listener = RemoveSubqueriesListener(ctx.depth())
@@ -314,11 +386,6 @@ class MySQLQueryProcessor(object):
             keywords.extend(column_keyword_function_listener.keywords)
             functions.extend(column_keyword_function_listener.functions)
 
-            # Let's make sure we have a select expression context
-            #  if not isinstance(column_keyword_function_listener.data[0][1],
-            #                    MySQLParser.Select_expressionContext):
-            #      continue
-
             # Does the subquery has an alias?
             try:
                 subquery_alias = subquery_aliases[ctx]
@@ -335,6 +402,8 @@ class MySQLQueryProcessor(object):
                 join_using, column_aliases =\
                 self._extract_instances(column_keyword_function_listener)
 
+            tables.extend([i[0][0] for i in select_list_tables])
+
             # Then we need to connect the column names s with tables and
             # databases
 
@@ -351,10 +420,6 @@ class MySQLQueryProcessor(object):
                     for b in budget:
                         if ref == b[1]:
                             ref_dict[ref] = b
-                            #  ref_found = True
-
-                #  if not ref_found:
-                    #  raise QueryError('Missing table reference %s.' % ref)
 
             if not len(select_list_table_references):
                 for table in select_list_tables:
@@ -393,12 +458,6 @@ class MySQLQueryProcessor(object):
                     if b[0] > current_depth:
                         bp.append(budget.pop(-1)[2])
 
-                # check if the join_column is in each sub select_list
-                #  for b in bp:
-                #  bcols = [i[0][2] for i in b]
-                #  if join_using[0][0][2] not in bcols and '*' not in bcols
-                #  raise QueryError('Missing join column `%s`.' %
-                #                         #  join_using[0][0][2])
                 budget.extend(join_columns)
 
             if subquery_alias is not None:
@@ -445,6 +504,7 @@ class MySQLQueryProcessor(object):
         self.keywords = list(set(keywords))
         self.functions = list(set(functions))
         self.display_columns = [(i[0], i[1]) for i in display_columns]
+        self.tables = tables
 
     @property
     def query(self):
@@ -454,14 +514,6 @@ class MySQLQueryProcessor(object):
         """
         return self._query
 
-    @property
-    def strict(self):
-        """
-        Get the strict flag.
-
-        """
-        return self._strict
-
     def _strip_query(self, query):
         return query.lstrip('\n').rstrip().rstrip(';') + ';'
 
